@@ -4,14 +4,17 @@ from __future__ import print_function
 
 try:
     import __builtin__ as builtins  # Python 2.7
-except:
+except ImportError:
     import builtins  # Python 3
 import argparse
 from datetime import datetime
 from importlib import import_module
+import random
+
+import numpy as np
 
 try:
-    import sntools  # when installing via pip, this should work
+    import sntools  # if sntools was installed via pip
 except ImportError:
     # if running this directly from the repo, modify `sys.path` to ensure all imports work
     import os
@@ -19,15 +22,16 @@ except ImportError:
     abs_dir = os.path.dirname(os.path.abspath(__file__))
     parent_dir = os.path.dirname(abs_dir)
     sys.path.append(parent_dir)
+    import sntools
 
 from sntools.channel import gen_evts
 from sntools.detectors import Detector, supported_detectors
 
 
-# mixing parameters from M. Tanabashi et al. (Particle Data Group), PRD 98 (2018), 030001
+# mixing parameters from P.A. Zyla et al. (Particle Data Group), Prog. Theor. Exp. Phys. 2020, 083C01 (2020)
 s12 = 0.307  # sin^2 theta_12
 c12 = 1 - s12
-s13 = 0.0212  # sin^2 theta_13
+s13 = 0.0218  # sin^2 theta_13
 c13 = 1 - s13
 
 # While exiting the supernova, neutrinos experience mass hierarchy-dependent
@@ -35,8 +39,7 @@ c13 = 1 - s13
 #   * original flavor at production (i.e. in input files from computer simulations),
 #   * mixing probability,
 #   * resulting flavor in detector.
-# See p. 266 of the 2018 Hyper-K Design Report (arXiv:1805.04163v1), but note
-# that this code includes theta_13 and assumes adiabatic transition (P_H = 0).
+# See the section "Treatment of Neutrino Flavour Conversion" in the documentation.
 mixings = {
     "noosc": (
         ("e", 1, "e"),
@@ -79,6 +82,7 @@ def main():
     distance = args.distance
     starttime = args.starttime if args.starttime else None
     endtime = args.endtime if args.endtime else None
+    seed = args.randomseed
     verbose = args.verbose
 
     if verbose:
@@ -90,12 +94,16 @@ def main():
         print("distance   =", distance)
         print("starttime  =", starttime)
         print("endtime    =", endtime)
+        print("randomseed =", seed)
         print("**************************************")
 
+    random.seed(seed)
+    np.random.seed(seed)
+
     # Take into account hierarchy-dependent flavor mixing and let channel.py
     # generate the actual events for each channel.
-    events_by_channel = {}
-    for channel in channels:
+    events = []
+    for channel in sorted(channels):
         mod_channel = import_module("sntools.interaction_channels." + channel)
         for (original_flv, scale, detected_flv) in mixings[hierarchy]:
             if detected_flv in mod_channel.possible_flavors:
@@ -111,13 +119,10 @@ def main():
                     % (channel, input, format, original_flv, scale, starttime, endtime, verbose)
                 if verbose:
                     print("Now executing:", cmd)
-                events_by_channel[(channel, original_flv, detected_flv)] = eval(cmd)
+                events.extend(eval(cmd))
 
-    # Collect events generated in all interaction channels.
-    events = [evt for evtlist in events_by_channel.values() for evt in evtlist]
-    events.sort(key=lambda evt: evt.time)  # sort events by time
-
-    # Write events to a nuance-formatted output file
+    # Sort events by time and write them to a nuance-formatted output file
+    events.sort(key=lambda evt: evt.time)
     with open(output, "w") as outfile:
         if verbose:  # write parameters to file as a comment
             outfile.write("# Generated on %s with the options:\n" % datetime.now())
@@ -163,16 +168,21 @@ def parse_command_line_options():
 
     default = 10.0
     parser.add_argument("--distance", type=float, default=default,
-                      help="Distance to supernova in kpc. Default: '%s'." % default)
+                        help="Distance to supernova in kpc. Default: '%s'." % default)
 
     parser.add_argument("--starttime", metavar="T", type=float,
-                      help="Start generating events at T milliseconds. Default: First time bin in input file.")
+                        help="Start generating events at T milliseconds. Default: First time bin in input file.")
 
     parser.add_argument("--endtime", metavar="T", type=float,
-                      help="Stop generating events at T milliseconds. Default: Last time bin in input file.")
+                        help="Stop generating events at T milliseconds. Default: Last time bin in input file.")
+
+    parser.add_argument("--randomseed", metavar="SEED", type=int,  # non-ints may not give reproducible results
+                        help="Integer used as a random number seed to reproducibly generate events. Default: None.")
 
     parser.add_argument("-v", "--verbose", action="count", help="Verbose output, e.g. for debugging. Off by default.")
 
+    parser.add_argument('--version', action='version', version='%(prog)s ' + sntools.__version__)
+
     return parser.parse_args()
 
 
